// Global variables
var gLC_Alerts;			// Alerts service
var gLC_Observer;		// Observer service
var gLC_Mediator;		// Window Mediator service
var gLC_ArcURL;

var gLC_Notifiables;	// Only feeds to be notified about
var gLC_Strings;		// Localized strings

// Primary LiveClick class
var LiveClick =
{
	// Constants
	// "Open Location", "View Feed", "Mark All Items as Read" or "(No unread items)", separator
	MENU_COUNT : 4,

	// Properties
	forceFeed : false,

	// Initialization code
	startup : function ()
	{
		// Initialize globals
		if ("@mozilla.org/alerts-service;1" in Components.classes)
			gLC_Alerts = Components.classes["@mozilla.org/alerts-service;1"].
							getService( Components.interfaces.nsIAlertsService );

		gLC_Observer = Components.classes["@mozilla.org/observer-service;1"].
						getService( Components.interfaces.nsIObserverService );

		gLC_Mediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].
						getService( Components.interfaces.nsIWindowMediator );

		if (!RDF) initServices();
		gLC_ArcURL = RDF.GetResource( gNC_NS + "URL" );

		// Get localized strings
		gLC_Strings = document.getElementById( "liveclick-strings" );

		// Retrieve list of feeds to be notified on
		gLC_Notifiables = NotifiableUtils.getList();

		var bm = document.getElementById( "bookmarks-menu" );
		if (bm)
		{
			// Listen for bookmarks menu events (added 0.2.1)
			bm.addEventListener( "popupshowing", this.onShowMenu, false );
			bm.addEventListener( "popuphidden", this.onHideMenu, false );
		}

		var bt = document.getElementById( "bookmarks-ptf" );
		if (bt)
		{
			// Listen for bookmark toolbar events (added 0.2.1)
			bt.addEventListener( "popupshowing", this.onShowMenu, false );
			bt.addEventListener( "popuphidden", this.onHideMenu, false );

			// Listen for new items on bookmarks toolbar (added 0.1.6)
			// Needed to draw icons on livemark buttons on startup and after changes
			bt.addEventListener( "DOMNodeInserted", LiveIconsUtils.iconifyButton, false );

			// Observe live bookmark updates (added 0.2.0)
			// Needed for monitors
			bt.database.AddObserver( LC_BMObserver );
		}

		var bc = document.getElementById( "bookmarks-chevron" );
		if (bc)
		{
			// Listen for bookmarks chevron events (added 0.2.1)
			bc.addEventListener( "popupshowing", this.onShowMenu, false );
			bc.addEventListener( "popuphidden", this.onHideMenu, false );
		}

		// Add LiveClick items to content area context menu (added 0.2.3)
		var cmContent = document.getElementById( "contentAreaContextMenu" );
		if (cmContent)
			cmContent.addEventListener( "popupshowing", this.editCMContent, false );

		// Add LiveClick items to bookmarks context menu (added 0.2.1)
		var cmBookmark = document.getElementById( "bookmarks-context-menu" );
		if (cmBookmark)
			cmBookmark.addEventListener( "popupshowing", this.editCMBookmark, false );

		// Override BookmarksMenu.validOpenInTabsMenuItem (added 0.2.0)
		BookmarksMenu.validOpenInTabsMenuItem = this.validOpenInTabs;

		// Observe changes to bookmark properties/options (added 0.2.0)
		// Needed to update list of notifiables from other windows
		gLC_Observer.addObserver( NotifiableObserver, "notifiables-add", false );
		gLC_Observer.addObserver( NotifiableObserver, "notifiables-remove", false );
		gLC_Observer.addObserver( NotifiableObserver, "notifiables-update", false );

		// Remove listeners/observers on window unload
		window.addEventListener( "unload", function() { LiveClick.shutdown(); }, false );
	},

	shutdown : function ()
	{
		// Forcibly close alert window if open
		// Otherwise, Firefox doesn't quit (=> memory leaks)
		if (LC_Alerter._alertVisible)
		{
			// Find alert window; probably a better way to do this
			var en = gLC_Mediator.getEnumerator( "" );
			while (en.hasMoreElements())
			{
				var w = en.getNext();
				if (w.location == "chrome://global/content/alerts/alert.xul")
				{
					w.close();
					break;
				}
			}
		}

		gLC_Observer.removeObserver( NotifiableObserver, "notifiables-add" );
		gLC_Observer.removeObserver( NotifiableObserver, "notifiables-remove" );
		gLC_Observer.removeObserver( NotifiableObserver, "notifiables-update" );

		var bt = document.getElementById( "bookmarks-ptf" );
		if (!bt) return;

		bt.database.RemoveObserver( LC_BMObserver );
	},

	// Extends the onClick behavior of a bookmark
	onClick : function (aEvent)
	{
		var theLivemark = aEvent.target;

		// Fix 0.1.5.2: Break out if clicking on a non-bookmark item
		// i.e. menuseparator, "Bookmark This Page," "Manage Bookmarks", etc.
		if (!theLivemark.id) return;

		var sLocal = theLivemark.localName;

		// Draw favicon if toolbarbutton (added 0.1.6)
		if (sLocal == "toolbarbutton")
			LiveIconsUtils.iconifyItem( theLivemark );

		var iButton = aEvent.button;

		var iOnLeft = LC_Prefs.getValue( "onLeftClick" );
		var iOnMiddle = LC_Prefs.getValue( "onMiddleClick" );
		var iOnAccel = LC_Prefs.getValue( "onAccelClick" );

		var iAction = -1;
		// -1 => Do browser default
		//  0 => Do nothing
		//  1 => Open location in current window/tab
		//  2 => Open location in new tab
		//  3 => View feed in current window/tab
		//  4 => View feed in new tab
		//  5 => Open all items in tab(s)
		//  6 => Mark all items as read
		//  7 => Open unread items in tab(s)
		//  9 => Reload live bookmark

		if (aEvent.ctrlKey && iOnAccel != -1 && iOnAccel != 5)
			// Set accel click action if
			//	1) action NOT browser default
			//	2) NOT "Open in Tabs" (default browser action)
			iAction = iOnAccel;
		else if (iButton == 0 && iOnLeft > 0 && sLocal != "toolbarbutton")
			// Set left click action if
			//	1) action NOT browser default and NOT do nothing (default browser action)
			//	2) bookmark NOT located directly on bookmarks toolbar
			iAction = iOnLeft;
		else if (iButton == 1 && iOnMiddle != -1 && iOnMiddle != 5)
			// Set middle click action if
			//	1) action NOT browser default
			//	2) NOT "Open in Tabs" (default browser action)
			iAction = iOnMiddle;

		// Continue if action NOT browser default
		if (iAction >= 0)
		{
			// Get container type
			var resource = RDF.GetResource( theLivemark.id );
			var sType = BookmarksUtils.resolveType( resource );

			// Process only on livemarks
			if (sType == "Livemark")
			{
				if (iAction == 0) return; // Do nothing

				// Even actions open bookmark in new tab
				// Odd actions open bookmark in current window/tab
				var sWhereToOpen = iAction % 2 == 0 ? "tab" : "current";

				// If shift key pressed, open in new window (added 0.2.0)
				if (aEvent.shiftKey) sWhereToOpen = "window";

				if (iAction == 1 || iAction == 2)
					// Open location
					this.loadBookmark( aEvent, "URL", sWhereToOpen );
				else if (iAction == 3 || iAction == 4)
					// View feed
					this.loadBookmark( aEvent, "FeedURL", sWhereToOpen );
				else if (iAction == 5)
					// Open all items in tabs
					this.loadGroupBookmark( aEvent );
				else if (iAction == 6)
					// Mark all items as read
					this.markAllRead( theLivemark.firstChild, true );
				else if (iAction == 7)
					// Open unread items in tabs
					this.loadNewGroup( theLivemark.firstChild );
				else if (iAction == 9)
					// Reload live bookmark
					this.reloadLivemark( theLivemark );

				// Bug forces us to close menus manually
				this.closeMenus( theLivemark );

				return;
			}
		}

		// If you reach this point, perform default middle click action(s)
		BookmarksMenu.loadBookmarkMiddleClick( aEvent )
	},

	loadBookmark : function (aEvent, sField, sWhereToOpen)
	{
		var target = aEvent.target;

		var selection;
		// If called from context menu
		if (BookmarksMenu._selection)
		{
			selection = BookmarksMenu._selection;
		}
		// If called from feed menu or as livemark click
		else
		{
			if (target.getAttribute( "class" ) == "liveclick-menuitem")
				target = target.parentNode.parentNode;

			var resource = RDF.GetResource( target.id );
			selection = BookmarksUtils.getSelectionFromResource( resource );
		}

		LiveClick.forceFeed = (sField == "FeedURL");
		var sURL = this.getBookmarkField( selection, sField );
		this.openURL( sURL, aEvent, sWhereToOpen );
	},

	loadBookmarkClick : function (aEvent, sField)
	{
		// Perform on middle click only
		if (aEvent.button != 1) return;

		this.loadBookmark( aEvent, sField, "tab" );
		this.closeMenus( aEvent.target );	// Bug forces us to close menus manually
	},

	// Open all in tabs
	loadGroupBookmark : function (aEvent)
	{
		var target = aEvent.target;
		var resource = RDF.GetResource( target.id );
		var selection = BookmarksUtils.getSelectionFromResource( resource );

		// Executed only on the first selection
		var sID = selection.item[0].Value;
		BookmarksCommand.openGroupBookmark( sID, "current" );
	},

	// Open unread feed items in tabs (added 0.2.1)
	// Revised 0.2.3: Get URLs through RDFC instead of XUL
	loadNewGroup : function (aPopup)
	{
		var iCount = 0;
		var urls = [];

		var theLivemark;
		// If called from context menu
		if (BookmarksMenu._selection)
		{
			var selection = BookmarksMenu._selection;
			theLivemark = selection.item[0];	// RDF Resource
		}
		// If called from feed menu or as livemark click
		else
		{
			theLivemark = RDF.GetResource( aPopup.parentNode.id );
		}

		RDFC.Init( BMDS, theLivemark );
		var children = RDFC.GetElements();
		while (children.hasMoreElements())
		{
			var rsrc = children.getNext().QueryInterface( kRDFRSCIID );
			var url = BMDS.GetTarget( rsrc, gLC_ArcURL, true );
			if (url)
			{
				var sURL = url.QueryInterface( Components.interfaces.nsIRDFLiteral ).Value;
				var uri = IOSVC.newURI( sURL, null, null );
				if (!LC_History.isVisited( uri ))
				{
					urls.push( sURL );
					iCount++;
				}
			}
		}

		// confirmOpenTabs in Firefox 2.0+ only
		try
		{
			// TODO: Close menus before prompting
			if (!BookmarksCommand._confirmOpenTabs( iCount )) return;
		}
		catch (e)
		{
			// Do Nothing
		}

		for (var j = 0; j < urls.length; j++)
			openUILinkIn( urls[j], "tab" );
	},

	// Mark selected item(s) as read/unread
	// TODO: Remove unread item from notifiable history
	markRead : function (aAsRead)
	{
		var selection = BookmarksMenu._selection;
		for (var i = 0; i < selection.length; i++)
		{
			var uri = this.getURI( selection.item[i] );
			if (aAsRead)
				LC_History.markAsRead( uri );
			else
				LC_History.markAsUnread( uri );
		}
	},

	// Mark all live bookmark feed items as read/unread
	// Revised 0.2.3: Get URLs through RDFC instead of XUL
	// Revised 0.2.3: Added argument for read/unread
	markAllRead : function (aPopup, aAsRead)
	{
		var theLivemark;
		// If called from context menu
		if (BookmarksMenu._selection)
		{
			var selection = BookmarksMenu._selection;
			theLivemark = selection.item[0];	// RDF Resource
		}
		// If called from feed menu or as livemark click
		else
		{
			theLivemark = RDF.GetResource( aPopup.parentNode.id );
		}

		RDFC.Init( BMDS, theLivemark );
		var children = RDFC.GetElements();
		while (children.hasMoreElements())
		{
			var rsrc = children.getNext().QueryInterface( kRDFRSCIID );
			var uri = this.getURI( rsrc );
			if (uri)
			{
				if (aAsRead)
					LC_History.markAsRead( uri );
				else
					LC_History.markAsUnread( uri );
			}
		}

		// Reset counts if marking all as unread
		if (!aAsRead)
		{
			LC_Monitor.resetCounts( theLivemark );

			// Flush if notifiable
			var sFeed = BookmarksUtils.getProperty( theLivemark, gNC_NS + "FeedURL" );
			if (gLC_Notifiables[sFeed] !== undefined)
				gLC_Notifiables[sFeed].flush();
		}
	},

	// Toggle monitor setting from bookmark context menu (added 0.2.3)
	toggleMonitor : function ()
	{
		var selection = BookmarksMenu._selection;
		if (selection.length != 1) return;

		var theLivemark = selection.item[0];	// RDF Resource

		var sFeed = BookmarksUtils.getProperty( theLivemark, gNC_NS + "FeedURL" );

		var bAdd = gLC_Notifiables[sFeed] === undefined;

		NotifiableUtils.updateFile( sFeed, bAdd );

		var sTopic = bAdd ? "add" : "remove";

		Components.classes["@mozilla.org/observer-service;1"].
			getService( Components.interfaces.nsIObserverService ).
			notifyObservers( null, "notifiables-" + sTopic, sFeed );
	},

	// Reload Live Bookmarks in folder (added 0.2.1)
	reloadLivemarks : function ()
	{
		var selection = BookmarksMenu._selection;
		if (selection.length != 1) return;

		var theFolder = selection.item[0];	// RDF Resource

		var sType = BookmarksUtils.resolveType( theFolder );
		if (sType != "Folder" && sType != "PersonalToolbarFolder")
			return;

		RDFC.Init( BMDS, theFolder );
		var children = RDFC.GetElements();
		while (children.hasMoreElements())
		{
			var rsrc = children.getNext().QueryInterface( kRDFRSCIID );
			var rtype = BookmarksUtils.resolveType( rsrc );
			if (rtype == "Livemark")
			{
				selection = BookmarksUtils.getSelectionFromResource( rsrc );
				BookmarksCommand.refreshLivemark( selection );
			}
		}
	},

	// Reload Live Bookmark (added 0.2.1)
	reloadLivemark : function (aLivemark)
	{
		var resource = RDF.GetResource( aLivemark.id );
		var selection = BookmarksUtils.getSelectionFromResource( resource );
		BookmarksCommand.refreshLivemark( selection );
	},

  	// Checks to see if popup menu can show Open in Tabs (added 0.2.0)
  	// Overrides BookmarksMenu.validOpenInTabsMenuItem
  	// Revised 0.2.3: Removed test for LiveClick empty item
	validOpenInTabs : function (aPopup)
	{
		var parent = RDF.GetResource( aPopup.parentNode.id );
		var sType = BookmarksUtils.resolveType( parent );

		// Return false if the parent is the bookmark menu or chevron
		if (sType != "Folder" && sType != "PersonalToolbarFolder" && sType != "Livemark")
			return false;

		// Return false if empty
		if (!aPopup.hasChildNodes()) return false;

		var iCount = 0;

		var node = aPopup.firstChild;
		while (node)
		{
			if (node.resource)
			{
				sType = BookmarksUtils.resolveType( node.id );
				if (sType == "Bookmark" || sType == "Livemark")
				{
					var sURL = BookmarksUtils.getProperty( node.id, gNC_NS + "URL" );
				  	// Revised 0.2.3: Discount about:livemark-loading, etc.
					if (sURL != "about:livemark-loading"
							&& sURL != "about:livemark-failed"
							&& sURL != "")
						iCount++;

					// Stop here and return true if at least 1 item has location
					// Revised 0.2.3: Changed from 2 to 1
					if (iCount >= 1) return true;
				}
			}

			node = node.nextSibling;
		}

		// Otherwise not valid in tabs
		return false;
	},

	// Extends the BookmarksMenu.onShowMenu method to alter feed menu
	// Fix 0.2.2: Removed call to showOpenInTabsMenuItem
	onShowMenu : function (aEvent)
	{
		var popup = aEvent.target;

		// Get container type
		var resource = RDF.GetResource( popup.parentNode.id );
		var sType = BookmarksUtils.resolveType( resource );

		// Edit menu if container is a livemark
		// Revised 0.2.0: Moved other tests to addMenuItems
		if (sType == "Livemark")
		{
			// Revised 0.2.3: Let others know that menu is open
			popup.parentNode.setAttribute( "isShowing", true );

			// Continue only if feed items have finished loading (or failed to load)
			var iLoading = LiveClick.getLoadingStatus( popup );
			if (iLoading == 0) return;

			// Revised 0.2.3: Style items only if loading successful
			if (iLoading == 1) LiveClick.styleFeedItems( popup );

			LiveClick.addMenuItems( popup );

			// Fix 0.1.5.1: Removed aDS argument from showEmptyItem
			BookmarksMenu.showEmptyItem( popup );
		}
		// Show favicons if a regular bookmarks menu (added 0.1.5)
		// Revised 0.1.6: Moved bShowFavIcons test to LiveIconsUtils
		else
			LiveIconsUtils.showInMenu( popup.parentNode );
	},

	// Extends the BookmarksMenu.onHideMenu method to hide LiveClick menu items
	// Revised 0.2.3: Test for classic empty-menuitem
	//  BookmarksMenu.hideEmptyItem only tests first and last nodes
	onHideMenu : function (aEvent)
	{
		var popup = aEvent.target;

		// Revised 0.2.3: Menu has closed; remove attribute
		if (popup.parentNode.hasAttribute( "isShowing" ))
			popup.parentNode.removeAttribute( "isShowing" );

		if (!popup.hasChildNodes()) return;

		var child = popup.lastChild;
		var removed = 0;
		while (child)
		{
			var sClass = child.getAttribute( "class" );
			if (sClass == "liveclick-menuitem"
				|| sClass == "liveclick-menuseparator"
				|| sClass == "liveclick-emptyitem"
				|| sClass == "empty-menuitem")
			{
				var prevchild = child.previousSibling;
				popup.removeChild( child );
				child = prevchild;
				removed++;
				// Break here if all LiveClick menu items removed
				if (removed == LiveClick.MENU_COUNT + 1) break;
			}
			else
			{
				child = child.previousSibling;
			}
		}
	},

	// Style live bookmark items to appear read/unread (added 0.2.0)
	styleFeedItems : function (aPopup)
	{
		var iReadStyle = LC_Prefs.getValue( "readStyle" );
		var sPrefix = (iReadStyle >= 0) ? iReadStyle.toString() + "_" : "";

		var nodeList = aPopup.childNodes;
		var iItemCount = nodeList.length;

		// Undocumented preference sets max number of items per feed menu
		var iFeedMax = LC_Prefs.getValue( "maxFeedItems" );
		iFeedMax = (iFeedMax > 0) ? iFeedMax : iItemCount;

		var iFeedCount = 0;
		var iFeedRead = 0;
		var iFeedHidden = 0;

		for (var i = 0; i < iItemCount; i++)
		{
			var item = nodeList[i];
			if (item.resource)
			{
				var sName = BookmarksUtils.getProperty( item.id, gNC_NS + "Name" );
				item.setAttribute( "tooltiptext", sName );

				if (iFeedCount < iFeedMax)
				{
					var resource = RDF.GetResource( item.id );
					var uri = this.getURI( resource );
					if (LC_History.isVisited( uri ))
					{
						item.setAttribute( "liveclick", sPrefix + "read" );
						iFeedRead++;

						// Mark as "read" again, otherwise old items eventually become "unread"
						LC_History.markAsRead( uri );
					}
					else
						item.setAttribute( "liveclick", sPrefix + "unread" );

					var sDisplay = document.defaultView.getComputedStyle( item, null ).
									getPropertyValue( "display" );

					if (sDisplay == "none") iFeedHidden++;
				}
				else
				{
					item.style.setProperty( "display", "none", "" );
					iFeedHidden++;
				}

				iFeedCount++;
			}
		}

		// Show placeholder when no unread feed items exist
		// Needed when reading style set to "Hide read items"
		if (iFeedCount - iFeedHidden == 0)
			this.showEmptyItem( aPopup );

		// Revised 0.2.3: Attach totals to livemark as attributes
		var nLivemark = aPopup.parentNode;

		nLivemark.setAttribute( "itemsTotal", iFeedCount );
		nLivemark.setAttribute( "itemsRead", iFeedRead );
		nLivemark.setAttribute( "itemsUnread", iFeedCount - iFeedRead );
		nLivemark.setAttribute( "itemsHidden", iFeedHidden );
	},

	// Add placeholder when no unread feed items exist
	// Revised 0.2.2: Insert before first child (rather than append to last)
	showEmptyItem : function (aPopup)
	{
		var node = document.createElementNS( gXUL_NS, "menuitem" );
		node.setAttribute( "label", gLC_Strings.getString( "emptyItem" ) );
		node.setAttribute( "class", "liveclick-emptyitem" );
		node.setAttribute( "disabled", "true" );

		// Revised 0.2.4: Style livemark to reflect read status
		var nBefore = aPopup.firstChild; // Top of menu
		aPopup.insertBefore( node, nBefore );
	},

	// Add LiveClick menu elements to menupopup (aPopup)
	// Revised 0.2.0: Moved tests here
	// Revised 0.2.2: Better bottom placement handling
	addMenuItems : function (aPopup)
	{
		var nodeList = aPopup.getElementsByAttribute( "class", "liveclick-menuitem" );
		if (nodeList.length > 0) return;	// Break here if menu already altered by LiveClick

		var nLivemark = aPopup.parentNode;

		// Test for empty menu
		var bHasItems = aPopup.hasChildNodes();

		// Revised 0.1.7: Always show left click option for livemarks on toolbar
		var bToolbar = nLivemark.localName == "toolbarbutton";
		var iOnLeft = LC_Prefs.getValue( "onLeftClick" );

		var bShowOpen = (LC_Prefs.getValue( "showPopupOpen" )
					|| (bToolbar && (iOnLeft == 1 || iOnLeft == 2)))
					&& this.hasLocation( nLivemark.id );
		var bShowFeed = LC_Prefs.getValue( "showPopupFeed" )
					|| (bToolbar && (iOnLeft == 3 || iOnLeft == 4));
		var bShowRead = (LC_Prefs.getValue( "showPopupRead" )
					|| (bToolbar && iOnLeft == 6))
					&& this.validOpenInTabs( aPopup );

		// Don't need to show any LiveClick elements; break here
		if (!bShowOpen && !bShowFeed && !bShowRead) return;

		// Create LiveClick menu elements
		// Separator
		var nSep1 = document.createElementNS( gXUL_NS, "menuseparator" );
		nSep1.setAttribute( "class", "liveclick-menuseparator" );

		var nOpen = this.createMenuItem( 1 ); // Open Location
		var nFeed = this.createMenuItem( 3 ); // View Feed

		// Revised 0.2.3: Show Mark All Unread if all items read
		var nRead;
		if (nLivemark.getAttribute( "itemsUnread" ) == 0)
			nRead = this.createMenuItem( 8 ); // Mark All Items as Unread
		else
			nRead = this.createMenuItem( 6 ); // Mark All Items as Read

		// Get placement preference
		var bShowAtTop = LC_Prefs.getValue( "showAtTop" ) == 1;

		var nBefore = null;
		if (bHasItems)
		{
			if (bShowAtTop)
				nBefore = aPopup.firstChild; // Top of menu
			else
			{
				// Find Open in Tabs separator
				var child = aPopup.lastChild;
				while (child)
				{
					var sClass = child.getAttribute( "class" );
					if (sClass == "openintabs-menuseparator")
					{
						nBefore = child;
						break;
					}
					child = child.previousSibling;
				}
			}
		}

		if (nBefore)
		{
			if (!bShowAtTop) aPopup.insertBefore( nSep1, nBefore );
			if (bShowOpen) aPopup.insertBefore( nOpen, nBefore );
			if (bShowFeed) aPopup.insertBefore( nFeed, nBefore );
			if (bShowRead) aPopup.insertBefore( nRead, nBefore );
			if (bShowAtTop) aPopup.insertBefore( nSep1, nBefore );
		}
		else
		{
			if (bHasItems) aPopup.appendChild( nSep1 );
			if (bShowOpen) aPopup.appendChild( nOpen );
			if (bShowFeed) aPopup.appendChild( nFeed );
			if (bShowRead) aPopup.appendChild( nRead );
		}
	},

	// Returns true if bookmark has a location (URL)
	hasLocation : function (aID)
	{
		var sURL = BookmarksUtils.getProperty( aID, gNC_NS + "URL" );
		return sURL != "";
	},

	// Make requested menu item element
	createMenuItem : function (aType)
	{
		var element = document.createElementNS( gXUL_NS, "menuitem" );
		element.setAttribute( "class", "liveclick-menuitem" );
		element.setAttribute( "disabled", false );

		// Open Location
		if (aType == 1)
		{
			element.setAttribute( "label", gLC_Strings.getString("open") );
			element.setAttribute( "accesskey", gLC_Strings.getString("openKey") );
			element.setAttribute( "oncommand", "LiveClick.loadBookmark(event, 'URL', '');" );
			element.setAttribute( "onclick", "LiveClick.loadBookmarkClick(event, 'URL');" );
		}
		// View Feed
		else if (aType == 3)
		{
			element.setAttribute( "label", gLC_Strings.getString("feed") );
			element.setAttribute( "accesskey", gLC_Strings.getString("feedKey") );
			element.setAttribute( "oncommand", "LiveClick.loadBookmark(event, 'FeedURL', '');" );
			element.setAttribute( "onclick", "LiveClick.loadBookmarkClick(event, 'FeedURL');" );
		}
		// Mark All Items as Read
		else if (aType == 6)
		{
			element.setAttribute( "label", gLC_Strings.getString("read") );
			element.setAttribute( "accesskey", gLC_Strings.getString("readKey") );
			element.setAttribute( "oncommand", "LiveClick.markAllRead(event.target.parentNode, true);" );
		}
		// Open Unread Items in Tabs
		else if (aType == 7)
		{
			element.setAttribute( "label", gLC_Strings.getString("groupUnread") );
			element.setAttribute( "accesskey", gLC_Strings.getString("groupUnreadKey") );
			element.setAttribute( "oncommand", "LiveClick.loadNewGroup(event.target.parentNode);" );
		}
		// Mark All Items as Read
		else if (aType == 8)
		{
			element.setAttribute( "label", gLC_Strings.getString("unread") );
			element.setAttribute( "accesskey", gLC_Strings.getString("unreadKey") );
			element.setAttribute( "oncommand", "LiveClick.markAllRead(event.target.parentNode, false);" );
		}
		// Reload Live Bookmarks (within a folder)
		else if (aType == 10)
		{
			element.setAttribute( "label", gLC_Strings.getString("reloadLivemarks") );
			element.setAttribute( "accesskey", gLC_Strings.getString("reloadLivemarksKey") );
			element.setAttribute( "oncommand", "LiveClick.reloadLivemarks();" );
		}
		// Mark as Read
		else if (aType == 16)
		{
			element.setAttribute( "label", gLC_Strings.getString("markRead") );
			element.setAttribute( "accesskey", gLC_Strings.getString("markKey") );
			element.setAttribute( "oncommand", "LiveClick.markRead(true);" );
		}
		// Mark as Unread
		else if (aType == 18)
		{
			element.setAttribute( "label", gLC_Strings.getString("markUnread") );
			element.setAttribute( "accesskey", gLC_Strings.getString("markKey") );
			element.setAttribute( "oncommand", "LiveClick.markRead(false);" );
		}
		// Monitor This Feed
		else if (aType == 20)
		{
			element.setAttribute( "label", gLC_Strings.getString("toggleMonitor") );
			element.setAttribute( "accesskey", gLC_Strings.getString("toggleMonitorKey") );
			element.setAttribute( "oncommand", "LiveClick.toggleMonitor();" );
			element.setAttribute( "type", "checkbox" );
		}
		// Menu Separator (type = 0)
		else
		{
			element = document.createElementNS( gXUL_NS, "menuseparator" );
			element.setAttribute( "class", "liveclick-menuseparator" );
		}

		return element;
	},

	// Adds LiveClick commands to content area context menus (added 0.2.3)
	editCMContent : function (aEvent)
	{
		var nMenu = document.getElementById( "liveclick-lbt-menu" );
		var nPage = document.getElementById( "liveclick-lbt-page" );
		var nLink = document.getElementById( "liveclick-lbt-link" );

		// Rehide all LiveClick commands
		gContextMenu.showItem( nMenu, false );
		gContextMenu.showItem( nPage, false );
		gContextMenu.showItem( nLink, false );

		if (gContextMenu.onLink)
		{
			var sLink = gContextMenu.linkURL ?
				gContextMenu.linkURL : gContextMenu.getLinkURL();
			nLink.setAttribute( "feed", sLink );
			gContextMenu.showItem( nLink, true );
		}
		// Revised 0.2.4: Don't show if right-clicking image
		else if (!gContextMenu.onImage)
		{
			// Retrieve feeds info
			var feeds = gBrowser.mCurrentBrowser.feeds;
			if (!feeds || feeds.length == 0) return;

			if (feeds.length == 1)
			{
				nPage.setAttribute( "feed", feeds[0].href );
				gContextMenu.showItem( nPage, true );
			}
			else
			{
				var nFeeds = document.getElementById( "liveclick-lbt-feeds" );

				// Reset feeds menu
				while (nFeeds.hasChildNodes())
					nFeeds.removeChild( nFeeds.lastChild );

				// Build the menu showing the available feed choices for preview
				for (var i = 0; i < feeds.length; i++)
				{
					var feedInfo = feeds[i];
					var nFeed = document.createElementNS( gXUL_NS, "menuitem" );
					var sLabel = feedInfo.title || feedInfo.href;
					nFeed.setAttribute( "label", sLabel );
					nFeed.setAttribute( "feed", feedInfo.href );
					nFeed.setAttribute( "tooltiptext", feedInfo.href );
					nFeed.setAttribute( "oncommand", "LiveClick.addLivemark(event);" );
					nFeeds.appendChild( nFeed );
				}

				gContextMenu.showItem( nMenu, true );
			}
		}
	},

	// Calls the Add Bookmark dialog with livemark arguments (added 0.2.3)
	addLivemark : function (aEvent)
	{
		var docshell = document.getElementById( "content" ).webNavigation;
		var dArgs =
		{
			name : docshell.document.title,
			description :
				BookmarksUtils.getDescriptionFromDocument( docshell.document ),
			url : docshell.currentURI.spec,
			charset : docshell.document.charset,
			bWebPanel : false,
			feedURL : aEvent.target.getAttribute( "feed" )
		};

		openDialog( "chrome://browser/content/bookmarks/addBookmark2.xul", "",
			   ADD_BM_DIALOG_FEATURES, dArgs );
	},

	// Adds LiveClick commands to bookmarks context menus (added 0.2.1)
	// Revised 0.2.3: Added commands for livemarks and livemark items
	// Remember! Use explicit class referencing instead of this.function()
	editCMBookmark : function (aEvent)
	{
		// Get bookmark whose context menu was called)
		var nBookmark = document.popupNode;					// As Node
		var theBookmark = RDF.GetResource( nBookmark.id );	// As Resource

		// Get context menu
		var popup = aEvent.target;

		var reference;
		var nSep1 = LiveClick.createMenuItem( 0 ); // Menu Separator
		var nSep2 = LiveClick.createMenuItem( 0 ); // Menu Separator

		// Get bookmark type
		var sType = BookmarksUtils.resolveType( theBookmark );
		if (sType == "Livemark")
		{
			var nOpen = LiveClick.createMenuItem( 1 ); // Open Location
			if (!LiveClick.hasLocation( nBookmark.id ))
				nOpen.setAttribute( "disabled", true );

			var nFeed = LiveClick.createMenuItem( 3 ); // View Feed
			var nMarkRead = LiveClick.createMenuItem( 6 ); // Mark All Items as Read
			var nMarkUnrd = LiveClick.createMenuItem( 8 ); // Mark All Items as Unread
			var nOpenUnrd = LiveClick.createMenuItem( 7 ); // Open Unread Items in Tabs

			if (nBookmark.getAttribute( "itemsUnread" ) == 0)
			{
				nOpenUnrd.setAttribute( "disabled", true );
				nMarkRead.setAttribute( "disabled", true );
			}

			if (nBookmark.getAttribute( "itemsRead" ) == 0)
				nMarkUnrd.setAttribute( "disabled", true );

			var nMonitor = LiveClick.createMenuItem( 20 ); // Monitor This Feed
			var sFeed = BookmarksUtils.getProperty( theBookmark, gNC_NS + "FeedURL" );
			var bMonitor = gLC_Notifiables[sFeed] !== undefined;
			nMonitor.setAttribute( "checked", bMonitor );

			// Get Open All in Tabs (should be first item)
			reference = popup.firstChild;
			var bHasItems = nBookmark.getAttribute( "itemsTotal" ) > 0;
			reference.setAttribute( "disabled", !bHasItems ); // Doesn't work?

			// Insert before Open All in Tabs
			popup.insertBefore( nOpen, reference );
			popup.insertBefore( nFeed, reference );
			popup.insertBefore( nSep1, reference );

			// Move pointer to separator after Open All in Tabs
			reference = reference.nextSibling;
			popup.insertBefore( nOpenUnrd, reference );
			popup.insertBefore( nSep2, reference );
			popup.insertBefore( nMarkRead, reference );
			popup.insertBefore( nMarkUnrd, reference );

			// Move pointer to Properties (should be last item)
			reference = popup.lastChild;
			popup.insertBefore( nMonitor, reference );
		}
		else if (sType == "Folder" || sType == "PersonalToolbarFolder")
		{
			var nReload = LiveClick.createMenuItem( 10 ); // Reload Live Bookmarks

			// Insert element before separator before Sort by Name
			for (i = popup.childNodes.length - 1; i >= 0; i--)
			{
				reference = popup.childNodes[i];
				if (reference.getAttribute( "label" ) == BookmarksUtils.getLocaleString( "cmd_bm_sortbyname" ))
					break;
			}

			popup.insertBefore( nReload, reference );
		}
		else if (sType == "Bookmark")
		{
			var parent = nBookmark.parentNode.parentNode;
			var rsrc = RDF.GetResource( parent.id );
			var rtype = BookmarksUtils.resolveType( rsrc );
			if (rtype == "Livemark")
			{
				var nMarkItem;

				var uri = LiveClick.getURI( theBookmark );
				if (LC_History.isVisited( uri ))
					nMarkItem = LiveClick.createMenuItem( 18 ); // Mark Item as Unread
				else
					nMarkItem = LiveClick.createMenuItem( 16 ); // Mark Item as Read

				// Insert elements before separator before Copy
				reference = popup.lastChild;
				reference = reference.previousSibling;

				popup.insertBefore( nSep1, reference );
				popup.insertBefore( nMarkItem, reference );
			}
		}
	},

	// Build feed button context menu (added 0.1.5)
	previewFeedMenu : function (aTarget)
	{
		// Retrieve feeds info
		var feeds = gBrowser.mCurrentBrowser.feeds;

		// No feeds, no menu
		if (feeds.length == 0) return false;

		// Reset menu
		while (aTarget.hasChildNodes())
			aTarget.removeChild( aTarget.lastChild );

		// Build the menu showing the available feed choices for preview
		for (var i = 0; i < feeds.length; i++)
		{
			var feedInfo = feeds[i];
			var nFeed = document.createElementNS( gXUL_NS, "menuitem" );
			var sTitle = feedInfo.title || feedInfo.href;
			var sLabel = gLC_Strings.getFormattedString( "previewFeed", [sTitle] );
			nFeed.setAttribute( "label", sLabel );
			nFeed.setAttribute( "feed", feedInfo.href );
			nFeed.setAttribute( "tooltiptext", feedInfo.href );
			aTarget.appendChild( nFeed );
		}

		return true;
	},

	// Load feed (added 0.1.5)
	// Revised 0.2.0: Temporarily override Firefox 2.0's feed handling to allow feed view
	// Revised 0.2.1: Split functions into previewFeed and previewFeedClick
	previewFeed : function (aEvent)
	{
  		var sURL = aEvent.target.getAttribute( "feed" );
  		this.forceFeedView();
		this.openURL( sURL, aEvent, '' );
	},

	// Handle middle-click independently of left-click (added 0.2.1)
	previewFeedClick : function (aEvent, aTarget)
	{
		// Perform on middle click only
		if (aEvent.button != 1) return;

		// Close popup manually
		aTarget.hidePopup();

		this.previewFeed( aEvent );
	},

	// Force feed view if Firefox 2.0+ (added 0.2.0)
	// Revised 0.2.3: Moved to its own function
	forceFeedView : function()
	{
		if ("@mozilla.org/browser/feeds/result-service;1" in Components.classes)
		{
			var feedService = Components.classes["@mozilla.org/browser/feeds/result-service;1"].
				getService( Components.interfaces.nsIFeedResultService);
			feedService.forcePreviewPage = true;
		}

		// Reset force feed view status
		LiveClick.forceFeed = false;
	},

	// Returns loading status of feed items menu (added 0.2.0)
	// Revised 0.2.3: Add test case for empty
	//  0 => Loading...
	//  1 => Finished loading
	//  2 => Failed to load
	//  3 => Empty folder
	getLoadingStatus : function (aPopup)
	{
		// No items exist so probably still loading
		if (!aPopup.hasChildNodes()) return 0;

		// At least 2 items exist at this point: Separator, Open in Tabs
		// More than 3 items => done loading; break here
		if (aPopup.childNodes.length > 3) return 1;

		var sLabel = aPopup.firstChild.getAttribute( "label" );
		if (sLabel == BookmarksUtils.getLocaleString( "BookmarksLivemarkLoading" ))
			return 0;
		else if (sLabel == BookmarksUtils.getLocaleString( "BookmarksLivemarkFailed" ))
			return 2;
		else if (sLabel == BookmarksUtils.getLocaleString( "emptyFolder" ))
			return 3;

		return 1;
	},

	getBookmarkField : function (aSelection, sField)
	{
		// Executed only on the first selection
		var sID = aSelection.item[0].Value;
		var sURL = BookmarksUtils.getProperty( sID, gNC_NS + sField );

		// Ask to use FeedURL if URL is empty (added 0.2.1)
		if (sField == "URL" && sURL == "")
		{
			var bPrompt = LC_Prefs.getValue( "promptNoLocation" );
			if (bPrompt)
			{
				var promptService =
					Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
					getService( Components.interfaces.nsIPromptService );

				var flags = promptService.BUTTON_TITLE_YES * promptService.BUTTON_POS_0 +
					promptService.BUTTON_TITLE_NO * promptService.BUTTON_POS_1;

				var bCheck = { value: true };

				var iButton = promptService.confirmEx(
					window, gLC_Strings.getString( "NoLocation" ),
					gLC_Strings.getString( "NoLocationMessage" ),
					flags, null, null, null,
					gLC_Strings.getString( "NoLocationCheck" ), bCheck );

				if (iButton != 0) return "";

				LC_Prefs.setPreference( "promptNoLocation", bCheck.value );
			}

			sField = "FeedURL";
			sURL = BookmarksUtils.getProperty( sID, gNC_NS + "FeedURL" );
			LiveClick.forceFeed = true;
		}

		return sURL;
	},

	// Retrieve URI from a RDF resource (added 0.2.3)
	getURI : function (aResource)
	{
		var uri = null;

		var url = BMDS.GetTarget( aResource, gLC_ArcURL, true );
		if (url)
		{
			var sURL = url.QueryInterface( Components.interfaces.nsIRDFLiteral ).Value;
			switch (sURL)
			{
				case "about:livemark-failed":
				case "about:livemark-loading":
					// Do Nothing
					break;
				default:
					uri = IOSVC.newURI( sURL, null, null );
					break;
			}
		}

		return uri;
	},

	// Open URL
	// Revised 0.2.1: Better integration with Tab Mix Plus bookmark click
	openURL : function (sURL, aEvent, sWhereToOpen)
	{
		if (sURL == "") return;

		// Let Tab Mix Plus decide where to open link if installed (added 0.2.1)
		// TMP uses BookmarksMenu.loadBookmark, which LiveClick bypasses
		// Note: This overrides LiveClick General preferences
		if (typeof( whereToOpenLinkTabmix ) == "function"
			&& typeof( prefStringTMBookmark ) == "string")
			sWhereToOpen = whereToOpenLinkTabmix( aEvent, prefStringTMBookmark );

		// Find target if none specified (based on event button, accel, etc.)
		if (sWhereToOpen == "")
			sWhereToOpen = whereToOpenLink( aEvent );

		// URL is from feed; force Firefox to view it without prompting
		if (this.forceFeed) this.forceFeedView();

		openUILinkIn( sURL, sWhereToOpen );

		if (content) content.focus();

		// Revised 0.2.3: preventBubble deprecated
		aEvent.stopPropagation();
	},

	// Open Options dialog
	// Revised 0.2.4: Made compatible with Macs
	//	Based on utilityOverlay.js openPreferences
	openOptions : function()
	{
		// Focus on dialog if already open
		var win = gLC_Mediator.getMostRecentWindow( "LiveClick:PrefWindow" );
		if (win)
			win.focus();
		// Otherwise open new dialog (nonmodal if Mac)
		else
		{
			var bApply = getBoolPref( "browser.preferences.instantApply", false );
			var sModal = bApply ? "dialog=no" : "modal";

			window.openDialog(
				"chrome://liveclick/content/options.xul",
				"LiveClickPrefs",
				"chrome,titlebar,toolbar,centerscreen," + sModal );
		}
	},

	// Clicking a bookmarks menu item leaves top level menu active
	// Based on fix from https://bugzilla.mozilla.org/show_bug.cgi?id=307796
	closeMenus : function (node)
	{
		BookmarksMenuDNDObserver.mCurrentDragOverTarget = null;
		BookmarksMenuDNDObserver.onDragCloseTarget();

		while ("tagName" in node)
		{
			if (node.tagName == "menupopup" || node.tagName == "popup")
				node.hidePopup();

			node.removeAttribute( '_moz-menuactive' );
			node = node.parentNode;
		}
	}
}

// Write a message to the JavaScript console for debugging purposes
function gLC_debug (aMessage)
{
	Components.classes["@mozilla.org/consoleservice;1"].
		getService( Components.interfaces.nsIConsoleService ).
		logStringMessage( aMessage );
}

// Prints list of an object's properties for debugging purposes
function gLC_props (aObject)
{
	var sBuffer = "";
	for (var el in aObject)
		sBuffer += el + ":\n" + aObject[el] + "\n\n";

	gLC_debug( sBuffer );
}


// Register LiveClick as an onLoad listener. To avoid getting called
// multiple times, register in non-capturing mode (3rd param = false).
window.addEventListener( "load", function() { LiveClick.startup(); }, false );