// LiveClick Monitor object
var LC_Monitor =
{
	process : function (aSource)
	{
		var theLivemark = RDF.GetResource( aSource.Value );

		var sType = BookmarksUtils.resolveType( theLivemark );
		if (sType != "Livemark") return;

		// Always reset feed item counts even if not notifiable
		LC_Monitor.resetCounts( theLivemark );

		// "Blink" open popup to work around displacement bug
		LC_Monitor.blinkPopup( theLivemark );

		// Stop here if not active browser window
		if (getBrowser() != gLC_Mediator.getMostRecentWindow( "navigator:browser" ).getBrowser())
			return;

		var sFeed = BookmarksUtils.getProperty( theLivemark, gNC_NS + "FeedURL" );

		// Do not process non-notifiable feeds
		if (gLC_Notifiables[sFeed] === undefined) return;

		// Stop processing if notifiable has no new items
		var iCount = gLC_Notifiables[sFeed].getCount();
		if (iCount == 0) return;

		// Check monitor action
		//  0 => Do nothing
		//  1 => Show alert message (default)
		//  2 => Autoload new items
		var iAction  = LC_Prefs.getValue( "monitorAction" );
		if (iAction == 0) return;	// No action; break here

		switch (iAction)
		{
			case 1:
				LC_Alerter.addAlert( theLivemark );
				break;
			case 2:
				LC_Autoloader.processNew( theLivemark );
				break;
		}
	},

	// Reset feed item counts (added 0.2.3)
	// Does it all at once after livemark loads instead of after each bookmark
	resetCounts : function (aLivemark)
	{
		var sFeed = BookmarksUtils.getProperty( aLivemark, gNC_NS + "FeedURL" );
		var bNotify = gLC_Notifiables[sFeed] !== undefined;

		var iCount = 0;
		var iRead = 0;

		RDFC.Init( BMDS, aLivemark );
		var children = RDFC.GetElements();
		while (children.hasMoreElements())
		{
			var rsrc = children.getNext().QueryInterface( kRDFRSCIID );
			var url = BMDS.GetTarget( rsrc, gLC_ArcURL, true );
			if (url)
			{
				var sURL = url.QueryInterface( Components.interfaces.nsIRDFLiteral ).Value;
				if (sURL != "" && sURL != "about:livemark-failed" && sURL != "about:livemark-loading")
				{
					var uri = IOSVC.newURI( sURL, null, null );

					if (!LC_History.isVisited( uri ))
					{
						// Add to list of new items
						if (bNotify) gLC_Notifiables[sFeed].addNew( uri )
					}
					else
					{
						// Mark as "read" again, otherwise old items eventually become "unread"
						LC_History.markAsRead( uri );
						iRead++;
					}

					iCount++;
				}
			}
		}

		// Attach totals to livemark as attributes
		var nLivemark = document.getElementById( aLivemark.Value );
		if (nLivemark)
		{
			nLivemark.setAttribute( "itemsTotal", iCount );
			nLivemark.setAttribute( "itemsRead", iRead );
			nLivemark.setAttribute( "itemsUnread", iCount - iRead );
			nLivemark.setAttribute( "itemsHidden", 0 );
		}
	},

	// "Blink" open popup to work around displacement bug (added 0.2.3)
	blinkPopup : function (aLivemark)
	{
		var nLivemark = document.getElementById( aLivemark.Value );
		if (!nLivemark) return;
		if (!nLivemark.hasAttribute( "isShowing" )) return;

		var popup = nLivemark.firstChild;
		popup.hidePopup();
		popup.showPopup();
	},

	addTab : function (aURL, aFore)
	{
		var newTab = window.getBrowser().addTab( aURL );
		if (aFore) window.getBrowser().selectedTab = newTab;
	},

	openNewest : function (aFeed, aFore)
	{
		var uri = gLC_Notifiables[aFeed].itemsNew[0];
		this.addTab( uri.spec, aFore );
	},

	openAllNew : function (aFeed)
	{
		var iCount = gLC_Notifiables[aFeed].getCount();
		for (var i = 0; i < iCount; i++)
		{
			var uri = gLC_Notifiables[aFeed].itemsNew[i];
			this.addTab( uri.spec );
		}
	}
}

// LiveClick BMDS observer object
var LC_BMObserver =
{
	onUnassert : function (aDS, aSource, aProperty, aTarget)
	{
		if (aProperty.Value == gNC_NS + "LivemarkLock")
			LC_Monitor.process( aSource );
	},

	// Revised 0.2.3: onAssert split into Monitor.process, Monitor.resetCount
	onAssert : function (aDS, aSource, aProperty, aTarget) {},
	onChange : function (aDS, aSource, aProperty, aOldTarget, aNewTarget) {},
	onMove : function (aDS, aOldSource, aNewSource, aProperty, aTarget) {},
	onBeginUpdateBatch : function (aDS) {},
	onEndUpdateBatch : function (aDS) {}
}

// LiveClick Alerter object
var LC_Alerter =
{
	_alerts : null,
	_alertVisible : false,
	_current : null,

	addAlert : function (aResource)
	{
		var sFeed = BookmarksUtils.getProperty( aResource, gNC_NS + "FeedURL" );
		var sName = BookmarksUtils.getProperty( aResource, gNC_NS + "Name" );
		var sURL = BookmarksUtils.getProperty( aResource, gNC_NS + "URL" );

		var iCount = gLC_Notifiables[sFeed].getCount();

		if (!this._alerts)
			this._alerts = new Array();

		for (var i = 0; i < this._alerts.length; i++)
		{
			// Alert already added; break here
			if (this._alerts[i].feed == sFeed
				&& this._alerts[i].name == sName
				&& this._alerts[i].url == sURL) return;
		}

		var alert = new Object();
		alert.feed = sFeed;
		alert.name = sName;
		alert.url = sURL;

		this._alerts.push( alert );

		if (!this._alertVisible)
			this.showAlerts();
	},

	showAlerts : function ()
	{
		if (this._alertVisible
			|| !this._alerts
			|| this._alerts.length == 0) return;

		// Need to shift here lest array grows w/o ever shrinking
		var alert = this._alerts.shift();

		// Stop here if alert service disabled (i.e. Macs)
		if (gLC_Alerts == null) return;

		var sFeed = alert.feed;
		var sName = alert.name;
		var sURL = alert.url == "" ? sFeed : alert.url;

		var iCount = gLC_Notifiables[sFeed].getCount();
		if (iCount == 0) return;

		this._alertVisible = true;
		this._current = alert;

		var sAlert = sName != "" ? sName + " " : "";
		sAlert = sAlert + "(" + iCount.toString() + ")";

		gLC_Alerts.showAlertNotification(
					"chrome://liveclick/skin/logo.png",
					gLC_Strings.getString( "alertNewFeeds" ),
					sAlert, true, "", this );
	},

	observe : function (aSubject, aTopic, aData)
	{
		if (aTopic == "alertclickcallback")
			this.onAlertClickCallback();
		else if (aTopic == "alertfinished")
			this.onAlertFinished();
	},

	onAlertClickCallback : function ()
	{
		var sFeed = this._current.feed;

		var iCount = gLC_Notifiables[sFeed].getCount();

		var iAction = iCount == 1 ?
			LC_Prefs.getValue( "alert1Action" ) : LC_Prefs.getValue( "alertNAction" );
		//  0 => Do Nothing
		//  1 => Open location in new tab
		//  2 => View feed in new tab
		//  3 => Open newest item in new tab
		//  4 => Open all new items in tab(s)

		if (iAction == 0) return;

		var bForeground = LC_Prefs.getValue( "alertForeground" );

		var sURL = this._current.url;

		switch (iAction)
		{
			case 1:
				LC_Monitor.addTab( sURL == "" ? sFeed : sURL, bForeground );
				break;
			case 2:
				LC_Monitor.addTab( sFeed, bForeground );
				break;
			case 3:
				LC_Monitor.openNewest( sFeed, bForeground );
				break;
			case 4:
				LC_Monitor.openAllNew( sFeed );
				break;
		}

		var bMarkRead = iCount == 1 ?
			LC_Prefs.getValue( "alert1Read" ) : LC_Prefs.getValue( "alertNRead" );

		if (bMarkRead)
		{
			// Mark all new items as "read"
			for (var i = 0; i < iCount; i++)
			{
				var uri = gLC_Notifiables[sFeed].itemsNew[i];
				LC_History.markAsRead( uri );
			}
		}
	},

	onAlertFinished : function ()
	{
		var sFeed = this._current.feed;

		// Revised 0.2.3: Test if feed is notifiable
		// Feed properties may be manually changed while alert visible
		if (gLC_Notifiables[sFeed] !== undefined)
			gLC_Notifiables[sFeed].freeze();

		this._alertVisible = false;
		this._current = null;
		this.showAlerts();
	}
}

// LiveClick Autoloader object
var LC_Autoloader =
{
	processNew : function (aResource)
	{
		var sFeed = BookmarksUtils.getProperty( aResource, gNC_NS + "FeedURL" );

		var iCount = gLC_Notifiables[sFeed].getCount();

		var iAction = iCount == 1 ?
			LC_Prefs.getValue( "autoload1Action" ) : LC_Prefs.getValue( "autoloadNAction" );
		//  0 => Do Nothing
		//  1 => Open location in new tab
		//  2 => View feed in new tab
		//  3 => Open newest item in new tab
		//  4 => Open all new items in tab(s)

		if (iAction == 0) return;

		var sName = BookmarksUtils.getProperty( aResource, gNC_NS + "Name" );
		var sURL = BookmarksUtils.getProperty( aResource, gNC_NS + "URL" );

		switch (iAction)
		{
			case 1:
				LC_Monitor.addTab( sURL == "" ? sFeed : sURL );
				break;
			case 2:
				LC_Monitor.addTab( sFeed );
				break;
			case 3:
				LC_Monitor.openNewest( sFeed );
				break;
			case 4:
				LC_Monitor.openAllNew( sFeed );
				break;
		}

		var bMarkRead = iCount == 1 ?
			LC_Prefs.getValue( "autoload1Read" ) : LC_Prefs.getValue( "autoloadNRead" );

		if (bMarkRead)
		{
			// Mark all new items as "read"
			for (var i = 0; i < iCount; i++)
			{
				var uri = gLC_Notifiables[sFeed].itemsNew[i];
				LC_History.markAsRead( uri );
			}
		}

		gLC_Notifiables[sFeed].freeze();
	}
}