// Notifiable object
function Notifiable ()
{
	this.itemsOld = [];
	this.itemsNew = [];

	this.addNew = addNew;
	this.getCount = getCount;
	this.freeze = freeze;
	this.flush = flush;

	function addNew (aURI)
	{
		for (var i = 0; i < this.itemsNew.length; i++)
			if (this.itemsNew[i] == aURI) return;

		this.itemsNew.push( aURI );
	}

	// Remove an item from notifiables history (added 0.2.3)
	// Not referenced yet, to be used with "Mark as Unread"
	function removeOld (aURI)
	{
		var index = -1;

		for (var i = 0; i < this.itemsOld.length; i++)
		{
			if (this.itemsOld[i] == aURI)
			{
				index = i;
				break;
			}
		}

		if (index >= 0)
			this.itemsOld.splice( index, 1 );
	}

	function getCount ()
	{
		var iCount = this.itemsNew.length;
		for (var i = 0; i < this.itemsNew.length; i++)
		{
			for (var j = 0; j < this.itemsOld.length; j++)
			{
				if (this.itemsOld[j].equals( this.itemsNew[i] ))
				{
					iCount--;
					break;
				}
			}
		}
		return iCount;
	}

	function freeze ()
	{
		this.itemsOld = this.itemsNew.slice();
		this.itemsNew = [];
	}

	// Reset notifiable history (added 0.2.3)
	function flush ()
	{
		this.itemsOld = [];
		this.itemsNew = [];
	}
}

// Notifiable observer object
var NotifiableObserver =
{
	observe : function (aSubject, aTopic, aData)
	{
		// Bookmark Properties dialog accepted with monitor added
		// Add Live Bookmark dialog accepted with monitor added
		if (aTopic == "notifiables-add")
		{
			try
			{
				if (gLC_Notifiables[aData] === undefined)
					gLC_Notifiables[aData] = new Notifiable();
			}
			catch (e) {}
		}
		// Bookmark Properties dialog accepted with monitor removed
		else if (aTopic == "notifiables-remove")
		{
			try
			{
				gLC_Notifiables = NotifiableUtils.refreshList( gLC_Notifiables );
			}
			catch (e) {}
		}
		// Options dialog accepted; update list of notifiables
		else if (aTopic == "notifiables-update")
		{
			try
			{
				gLC_Notifiables = NotifiableUtils.refreshList( gLC_Notifiables );
			}
			catch (e) {}
		}
	}
}

// Notifiable utils object
var NotifiableUtils =
{
	// Constants
	FILENAME : "liveclick-notify.txt",

	// Class properties
	path : null,

	getProfilePath : function ()
	{
		if (!this.path)
		{
			var sPath = Components.classes["@mozilla.org/file/directory_service;1"].
						 getService( Components.interfaces.nsIProperties ).
						 get( "ProfD", Components.interfaces.nsIFile ).path;

			this.path = sPath;
		}
		return this.path;
	},

	getFile : function ()
	{
		var file = Components.classes["@mozilla.org/file/local;1"].
					createInstance( Components.interfaces.nsILocalFile );
		file.initWithPath( this.getProfilePath() );
		file.append( this.FILENAME );
		return file;
	},

	getList : function ()
	{
		var feeds = {};

		var file = this.getFile();
		if (!file.exists()) return {};

		var fiStream = Components.classes["@mozilla.org/network/file-input-stream;1"].
						createInstance( Components.interfaces.nsIFileInputStream );

		fiStream.init( file, 0x01, 0444, null );
		fiStream.QueryInterface( Components.interfaces.nsILineInputStream );

		var line = {};
		var bLine;
		do
		{
			bLine = fiStream.readLine( line );
			if (line.value != "") feeds[line.value] = new Notifiable();
		} while (bLine)

		fiStream.close();

		return feeds;
	},

	// Retrieves a fresh list of notifiables (added 0.2.3)
	// Then copies history (if any) from existing list
	// Revised 0.2.4: Use argument instead of gLC_Notifiables
	refreshList : function (aFeeds)
	{
		var feedsNew = this.getList();

		for (var sFeed in feedsNew)
		{
			if (aFeeds[sFeed] !== undefined)
				feedsNew[sFeed] = aFeeds[sFeed];
		}

		return feedsNew;
	},

	writeFromArray : function (aFeeds)
	{
		var sData = aFeeds.length > 0 ? aFeeds.join( "\n" ) : "";

		var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                         createInstance( Components.interfaces.nsIFileOutputStream );

		foStream.init( this.getFile(), 0x02 | 0x08 | 0x20, 0664, 0 ); // write, create, truncate
		foStream.write( sData, sData.length );
		foStream.close();
	},

	updateFile : function (aFeedURL, aAdd)
	{
		var file = this.getFile();

		// Non-existent file; nothing to remove
		if (!file.exists() && !aAdd) return;

		var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                         createInstance( Components.interfaces.nsIFileOutputStream );

        // Create file with 1 notifiable
        if (!file.exists())
        {
			foStream.init( file, 0x02 | 0x08, 0664, 0 ); // write, create
			foStream.write( aFeedURL, aFeedURL.length );
			foStream.close();
			return;
		}

		var fiStream = Components.classes["@mozilla.org/network/file-input-stream;1"].
						createInstance( Components.interfaces.nsIFileInputStream );

		fiStream.init( file, 0x01, 0444, null );
		fiStream.QueryInterface( Components.interfaces.nsILineInputStream );

		var sData = "";
		var bOkayToAdd = true;

		var line = {};
		var bLine;
		do
		{
			bLine = fiStream.readLine( line );
			if (line.value == aFeedURL && aAdd)
				bOkayToAdd = true;
			else if (line.value != aFeedURL)
				sData = sData + line.value + "\n";
		} while (bLine && bOkayToAdd)

		fiStream.close();

		// Feed already a notifiable; no need to edit file
		if (!bOkayToAdd) return;

		if (aAdd) sData = sData + aFeedURL + "\n";

		foStream.init( file, 0x02 | 0x08 | 0x20, 0664, 0 ); // write, create, truncate
		foStream.write( sData, sData.length );
		foStream.close();
	}
}