const LC_GUID = "{d166ee2a-36bb-4f33-aff7-e85f912df509}";
const LC_ENGINE = 1;


var kRDFRSCIID, kRDFLITIID, kBMSVCIID;
var gNC_NS, gWEB_NS, gRDF_NS, gXUL_NS;
var RDF, RDFC, RDFCU, BMDS, BMSVC;


function initServices ()
{
	kRDFRSCIID	= Components.interfaces.nsIRDFResource;
	kRDFLITIID	= Components.interfaces.nsIRDFLiteral;
	kBMSVCIID	= Components.interfaces.nsIBookmarksService;

	gNC_NS		= "http://home.netscape.com/NC-rdf#";
	gWEB_NS		= "http://home.netscape.com/WEB-rdf#";
	gRDF_NS		= "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	gXUL_NS		= "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

	RDF		= Components.classes["@mozilla.org/rdf/rdf-service;1"].
				getService( Components.interfaces.nsIRDFService );
	RDFC	= Components.classes["@mozilla.org/rdf/container;1"].
				createInstance( Components.interfaces.nsIRDFContainer );
	RDFCU	= Components.classes["@mozilla.org/rdf/container-utils;1"].
				getService( Components.interfaces.nsIRDFContainerUtils );
	BMDS	= Components.classes["@mozilla.org/rdf/datasource;1?name=bookmarks"].
				getService( Components.interfaces.nsIRDFDataSource );
	BMSVC	= BMDS.QueryInterface( kBMSVCIID );
}

function setMinutes ()
{
	var seconds = document.getElementById( "refreshTime" ).value;
	seconds = (seconds != null) ? parseInt( seconds ) : 3600;
	return seconds / 60;
}

function setSeconds ()
{
	var minutes = document.getElementById( "minutesBox" ).value;
	minutes = (minutes != "") ? parseFloat( minutes ) : 0;
	if (minutes <= 0) return 0;
	return minutes * 60;
}

function getFeeds ()
{
	if (!RDF) initServices();

	var folders = [ RDF.GetResource( "NC:BookmarksRoot" ) ];
	var feeds = [];

	while (folders.length > 0)
	{
		RDFC.Init( BMDS, folders.shift() );

		var elements = RDFC.GetElements();
		while (elements.hasMoreElements())
		{
			var element = elements.getNext();
			element.QueryInterface( Components.interfaces.nsIRDFResource );

			var sType = BMUtils.resolveType( element, BMDS );

			if (sType == "Livemark")
			{
				var resource = RDF.GetResource( element.Value );
				var sName = BMUtils.getProperty( resource, gNC_NS + "Name" );
				var sFeed = BMUtils.getProperty( resource, gNC_NS + "FeedURL" );

				var feedObject =
				{
					"name" : sName,
					"feed" : sFeed
				};

				feeds.push( feedObject );
			}
			else if (sType == "Folder")
				folders.push( element );
		}
	}

	populateFeedList( feeds );
}

function populateFeedList (aFeeds)
{
	if (aFeeds.length == 0) return;
	aFeeds.sort( compareFeeds );

	var listBox = document.getElementById( "feeds" );
	var notifiables = NotifiableUtils.getList();

	for (var i = 0; i < aFeeds.length; i++)
	{
		var sName = aFeeds[i].name;
		var sFeed = aFeeds[i].feed;

		if (sName == "") sName = sFeed;

		if (sFeed != "")
		{
			var li = document.createElementNS( gXUL_NS, "listitem" );
			li.setAttribute( "value", sFeed );
			li.setAttribute( "label", sName );
			li.setAttribute( "tooltiptext", sName );
			li.setAttribute( "type", "checkbox" );

			if (typeof notifiables[sFeed] != "undefined")
				li.setAttribute( "checked", true );

			listBox.appendChild( li );
		}
	}
}

function compareFeeds (a, b)
{
	var sValueA = a.name != b.name ? a.name : a.feed;
	var sValueB = a.name != b.name ? b.name : b.feed;

	if (sValueA.toLowerCase() < sValueB.toLowerCase())
		return -1;
	else if (sValueA.toLowerCase() > sValueB.toLowerCase())
		return 1;

	return 0;
}

// Called when prefwindow is closed
// Needed to saveFeeds when instant apply enabled (i.e. Macs, Linux)
function closeDialog ()
{
	var bApply = Components.classes["@mozilla.org/preferences-service;1"].
					getService( Components.interfaces.nsIPrefService ).
					getBranch( "browser.preferences." ).
					getBoolPref( "instantApply" );

	if (bApply) saveFeeds();
}

function saveFeeds ()
{
	var listBox = document.getElementById( "feeds" );
	if (!listBox.hasChildNodes()) return;

	var notifiables = [];

	for (var i = 0; i < listBox.childNodes.length; i++)
	{
		var li = listBox.childNodes[i];
		if (li.getAttribute( "checked" ))
		{
			var sFeed = li.getAttribute( "value" );
			notifiables.push( sFeed );
		}
	}

	NotifiableUtils.writeFromArray( notifiables );

	// Tell browser windows to update list of notifiables
	Components.classes["@mozilla.org/observer-service;1"].
		getService( Components.interfaces.nsIObserverService ).
		notifyObservers( null, "notifiables-update", "" );

	return;
}

function defaultSettings()
{
	var file = Components.classes["@mozilla.org/extensions/manager;1"].
					getService( Components.interfaces.nsIExtensionManager ).
					getInstallLocation( LC_GUID ).
					getItemLocation( LC_GUID );

	file.append( "default-settings.xml" );
	restoreFromFile( file );

	// Get localized strings
	var strings = document.getElementById( "liveclick-strings" );
	alert( strings.getString( "SettingsDefault" ) );
}

function importSettings ()
{
	var fp = Components.classes["@mozilla.org/filepicker;1"].
				createInstance( Components.interfaces.nsIFilePicker );

	fp.init( window, null, fp.modeOpen );
	fp.appendFilters( fp.filterXML );
	fp.defaultExtension = "xml";
	fp.defaultString = "liveclick-settings";

	if (fp.show() != fp.returnCancel)
	{
		restoreFromFile( fp.file );

		// Get localized strings
		var strings = document.getElementById( "liveclick-strings" );
		alert( strings.getString( "SettingsImport" ) );
	}
}

function restoreFromFile (aFile)
{
	if (!aFile.exists()) return;

	var fiStream = Components.classes["@mozilla.org/network/file-input-stream;1"].
					createInstance( Components.interfaces.nsIFileInputStream );

	var siStream = Components.classes["@mozilla.org/scriptableinputstream;1"].
					createInstance( Components.interfaces.nsIScriptableInputStream );

	fiStream.init( aFile, 0x01, 0444, null );
	siStream.init( fiStream );

	var input = siStream.read( fiStream.available() );

	siStream.close();
	fiStream.close();

	var parser = new DOMParser();

	var doc = parser.parseFromString( input, "text/xml" );

	// Check for valid settings file
	var meta = doc.documentElement.getElementsByTagName( "meta" );
	if (meta.length != 1) return;

	var guid = meta[0].getElementsByTagName( "ext-guid" );
	if (guid.length != 1) return;
	if (guid[0].textContent != LC_GUID) return;

	var engine = meta[0].getElementsByTagName( "save-engine" );
	var iEngine = engine.length == 1 ? engine[0].textContent : LC_ENGINE;

	var i, sID, sValue;

	// Restore preference values
	if (doc.documentElement.getElementsByTagName( "prefs" ).length > 0)
	{
		var prefs = doc.documentElement.
			getElementsByTagName( "prefs" )[0].getElementsByTagName( "pref" );

		for (i = 0; i < prefs.length; i++)
		{
			if (prefs[i].hasAttribute( "id" ))
			{
				sID = prefs[i].getAttribute( "id" );
				sValue = prefs[i].textContent;

				var pref = document.getElementById( sID );
				if (pref)
				{
					if (pref.type == "bool")
						pref.value = sValue == "true";
					else
						pref.value = sValue;
				}
			}
		}
	}

	// Restore monitors
	if (doc.documentElement.getElementsByTagName( "monitors" ).length > 0)
	{
		var listBox = document.getElementById( "feeds" );

		// First uncheck all items
		for (i = 0; i < listBox.getRowCount(); i++)
			listBox.getItemAtIndex(i).checked = false;

		// Then check all matching feeds from settings file
		var monitors = doc.documentElement.
			getElementsByTagName( "monitors" )[0].getElementsByTagName( "monitor" );

		for (i = 0; i < monitors.length; i++)
		{
			if (monitors[i].hasAttribute( "id" ))
			{
				sID = monitors[i].getAttribute( "id" );
				var li = listBox.getElementsByAttribute( "value", sID );

				for (var j = 0; j < li.length; j++)
					li[j].checked = true;
			}
		}
	}
}

function exportSettings ()
{
	var fp = Components.classes["@mozilla.org/filepicker;1"].
				createInstance( Components.interfaces.nsIFilePicker );

	fp.init( window, null, fp.modeSave );
	fp.appendFilters( fp.filterXML );
	fp.defaultExtension = "xml";
	fp.defaultString = "liveclick-settings";

	if (fp.show() != fp.returnCancel)
	{
		var sXML = '<?xml version="1.0"?>\n';
		sXML += '<liveclick-settings>';

		sXML += makeXMLMeta();
		sXML += makeXMLPrefs();
		sXML += makeXMLMonitors();

		sXML += '</liveclick-settings>';

		var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                         createInstance( Components.interfaces.nsIFileOutputStream );

		var file = fp.file;
		foStream.init( file, 0x02 | 0x08 | 0x20, 0664, 0 ); // write, create, truncate
		foStream.write( sXML, sXML.length );
		foStream.close();
	}
}

function makeXMLMeta ()
{
	var sXML = '<meta>\n';

	var sVersion = "?";
	try
	{
		var em = Components.classes["@mozilla.org/extensions/manager;1"].
			getService( Components.interfaces.nsIExtensionManager );

		if ('getItemForID' in em)
		{
			var ext = em.getItemForID( LC_GUID );
			if (ext) sVersion = ext.version;
		}
	}
	catch (e)
	{}

	sXML += '<ext-guid>' + LC_GUID + '</ext-guid>\n';
	sXML += '<ext-version>' + sVersion + '</ext-version>\n';
	sXML += '<save-engine>' + LC_ENGINE + '</save-engine>\n';

	sXML += '</meta>\n';

	return sXML;
}

function makeXMLPrefs ()
{
	var sXML = '<prefs>\n';

	var prefs = document.getElementsByTagName( "preference" );
	for (var i = 0; i < prefs.length; i++)
	{
		var p = prefs[i];
		var sNode = '<pref id="' + p.id + '">' + p.value + '</pref>';
		sXML += sNode + "\n";
	}

	sXML += '</prefs>\n';

	return sXML;
}

function makeXMLMonitors ()
{
	var sXML = '<monitors>\n';

	var listBox = document.getElementById( "feeds" );

	for (var i = 0; i < listBox.getRowCount(); i++)
	{
		var li = listBox.getItemAtIndex(i);
		if (li.checked)
		{
			var sNode = '<monitor id="' + li.value + '">';
			sNode += '<custom>0</custom>';
			sNode += '</monitor>';
			sXML += sNode + "\n";
		}
	}

	sXML += '</monitors>\n';

	return sXML;
}

var BMUtils =
{
	getProperty : function (aInput, aArcURI)
	{
		var node;
		var arc  = RDF.GetResource( aArcURI );
		if (typeof( aInput ) == "string")
			aInput = RDF.GetResource( aInput );

		node = BMDS.GetTarget( aInput, arc, true );

		try
		{
			return node.QueryInterface( kRDFRSCIID ).Value;
		}
		catch (e)
		{
			return node ? node.QueryInterface( kRDFLITIID ).Value : "";
		}
	},

	resolveType : function (aResource)
	{
		var type = this.getProperty( aResource, gRDF_NS + "type");
		if (type != "") type = type.split( "#" )[1];
		return type;
	}
}

/*
// Write a message to the JavaScript console for debugging purposes
function gLC_writeToConsole (aMessage)
{
	Components.classes["@mozilla.org/consoleservice;1"].
		getService( Components.interfaces.nsIConsoleService ).
		logStringMessage( aMessage );
}
*/